/**
 * Взято из exmo_front
 * Использует Sockette, он должен быть в глобальной видимости
 */
(() => {
  // Копии функций из immutable, упрощающие чтение кода
  const update = (object, key, updateFn) => {
    const { [key]: value, ...objWithoutProperty } = object;

    return {
      ...objWithoutProperty,
      [key]: updateFn(value),
    };
  };
  const remove = (object, key) => {
    const { [key]: _, ...objWithoutProperty } = object;

    return objWithoutProperty;
  };

  const initPublicWS = (apiWSPublic) => {
    let topicListenersMap = {};
    let isConnected = false;

    const socket = new Sockette(apiWSPublic, {
      timeout: 10000,
      maxAttempts: Infinity,
      onmessage: (event) => {
        const data = JSON.parse(event.data);
        const messageTopic = data.topic;

        if (messageTopic && topicListenersMap[messageTopic]) {
          topicListenersMap[messageTopic].forEach((listener) => listener(data));
        }
      },
      onopen: () => {
        isConnected = true;

        const topics = Object.keys(topicListenersMap);

        if (topics.length) {
          // При реконнекте подписка заново
          socket.send(
            JSON.stringify({
              topics,
              method: "subscribe",
            })
          );
        }
      },
      onreconnect: () => {
        isConnected = false;
      },
      onclose: () => {
        isConnected = false;
      },
    });

    const publicWS = {
      subscribe: ({ topics, onMessage }) => {
        let topicsToSubscribe = [];

        topics.forEach((topic) => {
          if (!topicListenersMap[topic]) {
            topicListenersMap = { ...topicListenersMap, [topic]: [onMessage] };
            topicsToSubscribe = [...topicsToSubscribe, topic];
          } else {
            topicListenersMap = update(topicListenersMap, topic, (listeners) =>
              listeners.every((listener) => listener !== onMessage)
                ? [...listeners, onMessage]
                : listeners
            );
          }
        });

        // При подключении, подписки отошлются автоматически
        if (isConnected && topicsToSubscribe.length) {
          socket.send(
            JSON.stringify({
              method: "subscribe",
              topics: topicsToSubscribe,
            })
          );
        }
      },
      unsubscribe: ({ topics, onMessage }) => {
        topics.forEach((topic) => {
          let topicsToUnsubscribe = [];
          const topicListeners = topicListenersMap[topic];

          if (topicListeners) {
            if (
              topicListeners.length === 1 &&
              topicListeners[0] === onMessage
            ) {
              topicListenersMap = remove(topicListenersMap, topic);
              topicsToUnsubscribe = [...topicsToUnsubscribe, topic];
            } else {
              topicListenersMap = update(
                topicListenersMap,
                topic,
                (listeners) =>
                  listeners.filter((listener) => listener !== onMessage)
              );
            }
          }

          // При отключении, подписки отвалятся автоматически
          if (isConnected && topicsToUnsubscribe.length) {
            socket.send(
              JSON.stringify({
                method: "unsubscribe",
                topics: topicsToUnsubscribe,
              })
            );
          }
        });
      },
    };

    return publicWS;
  };

  window.initPublicWS = initPublicWS;
})();
