var Validator = {

    rules: {
        pair: /^[a-z0-9]+_[a-z0-9]+$/i
        , para: /^[a-z0-9]+_[a-z0-9]+$/i
        , period: /^[\w]+$/
        , type: /^[\w]+$/
        , lang: /^[a-z]+$/i
        , ref: /^[\d]+$/
    }

    /**
     * Check param, only if it has regex rule
     * @param string name
     * @param value
     * @return bool
     */
    , isValid: function (name, value) {
        if (typeof this.rules[name] === 'undefined') {
            throw new Error('Validator: param "' + name + '" does not have validate rule');
        }
        var isValid = this.rules[name].test(value);
        if (!isValid && typeof console.error === 'function') {
            console.error(
                'Validator: Invalid param "' + name + '", ' +
                'value "' + value + '",' +
                ' rule "' + this.rules[name] + '"'
            );
        }
        return isValid;
    }

    /**
     * Get value, if it not be empty and valid
     * @param string name
     * @param mixed data
     * @param defaultValue
     * @returns {*}
     */
    , getOrDefault: function (name, data, defaultValue) {
        var value = typeof data === 'object' ? data[name] : data;
        if (!value || !this.isValid(name, value)) {
            return defaultValue;
        }
        return value;
    }

    /**
     * Get param from URL with validate
     * @param string name
     * @param defaultValue
     * @returns {*}
     */
    , getFromUrl: function (name, defaultValue) {

        var results = new RegExp('[\\?&]' + name + '=([^&#]*)').exec(window.location.href);
        var value;
        if (results) {
            value = results[1];
        }

        return this.getOrDefault(name, value, defaultValue);
    }
};